@@ -6,12 +6,14 @@ import json
 import locale
 import sys
 import time
+import netifaces
 from datetime import datetime, timedelta
 from dateutil.tz import tzutc
 from argcomplete.completers import ChoicesCompleter, FilesCompleter
 from ec2_utils.instance_info import info
-from ec2_utils import logs, ebs, s3, interface, instance_info, utils
+from ec2_utils import block_devices, ebs, instance_info, interface, logs, utils, ecs
 from ec2_utils.s3 import prune_s3_object_versions
+from ec2_utils.utils import best_effort_stacks
 from threadlocal_aws import is_ec2, region as client_region
 
 SYS_ENCODING = locale.getpreferredencoding()
@@ -222,6 +224,26 @@ def detach_volume():
     else:
         parser.error("Only makes sense on an EC2 instance")
 
+def ecs_private_ip():
+    """ Get the private IP address of the container. Represents an ENI in the case of awsvpc networking
+    and the private interface of the EC2 instance in the case of host networkin"""
+    parser = _get_parser()
+    argcomplete.autocomplete(parser)
+    args = parser.parse_args()
+    print(ecs.get_private_ip())
+
+def first_ext_ip():
+    """ Get the first IP address attached to the instance that is not localhost """
+    parser = _get_parser()
+    argcomplete.autocomplete(parser)
+    args = parser.parse_args()
+    for iface in netifaces.interfaces():
+        addresses = netifaces.ifaddresses(iface)
+        if netifaces.AF_INET in addresses and addresses[netifaces.AF_INET] and "addr" in addresses[netifaces.AF_INET][0] and not addresses[netifaces.AF_INET][0]["addr"].startswith("127."):
+            print(addresses[netifaces.AF_INET][0]["addr"])
+            return
+    sys.exit(1)
+
 def get_tag():
     """ Get the value of a tag for an ec2 instance
     """
@@ -267,6 +289,15 @@ def instance_id():
     else:
         parser.error("Only makes sense on an EC2 instance")
 
+def largest_unmounted_device():
+    """ Get the largest block device that is currently not mounted """
+    parser = _get_parser()
+    argcomplete.autocomplete(parser)
+    parser.parse_args()
+    devs = block_devices.linux_unmounted_block_devices_largest_first()
+    if devs:
+        print(devs[0][0])
+
 def latest_snapshot():
     """Get the latest snapshot with a given tag
     """
@@ -351,13 +382,17 @@ def list_local_interfaces():
     argcomplete.autocomplete(parser)
     args = parser.parse_args()
     to_print={}
-    for iface in interface.get_network_interfaces():
+    for iface in netifaces.interfaces():
+        addresses = netifaces.ifaddresses(iface)
         if args.json:
-            to_print[iface.name] = { "index": iface.index,
-                                     "ipv4":  iface.addresses.get(interface.AF_INET),
-                                     "ipv6": iface.addresses.get(interface.AF_INET6) }
+            if not iface in to_print:
+                to_print[iface] = {}
+            if netifaces.AF_INET in addresses:
+                to_print[iface]["ipv4"] = addresses[netifaces.AF_INET]
+            if netifaces.AF_INET6 in addresses:
+                to_print[iface]["ipv6"] = addresses[netifaces.AF_INET6]
         else:
-            print(iface)
+            print(iface + ": " + str(addresses))
     if to_print:
         print(json.dumps(to_print, indent=2))
 
@@ -515,9 +550,10 @@ def register_private_dns():
     parser.add_argument("hosted_zone", help="The name of the hosted zone to update")
     parser.add_argument("-t", "--ttl", help="Time to live for the record. 60 by default",
                         default="60")
+    parser.add_argument("-p", "--private-ip", help="Private IP address to register")
     argcomplete.autocomplete(parser)
     args = parser.parse_args()
-    interface.register_private_dns(args.dns_name, args.hosted_zone, ttl=args.ttl)
+    interface.register_private_dns(args.dns_name, args.hosted_zone, ttl=args.ttl, private_ip=args.private_ip)
 
 def snapshot_from_volume():
     """ Create a snapshot of a volume identified by it's mount path
@@ -558,7 +594,7 @@ def stack_params_and_outputs():
                                                   " one parameter required")
     parser.add_argument("-s", "--stack-name", help="The name of the stack to show",
                         default=info().stack_name()).completer = \
-        ChoicesCompleter(_best_effort_stacks())
+        ChoicesCompleter(best_effort_stacks())
     argcomplete.autocomplete(parser)
     args = parser.parse_args()
     resp, _ = instance_info.stack_params_and_outputs_and_stack(stack_name=args.stack_name)
@@ -570,12 +606,6 @@ def stack_params_and_outputs():
     else:
         print(json.dumps(resp, indent=2))
 
-def _best_effort_stacks():
-    try:
-        return stacks()
-    except:
-        return []
-
 def subnet_id():
     """ Get subnet id for instance
     """
