@@ -310,21 +310,7 @@ def detach_volume(mount_path, delete_volume=False, volume_id=None):
     instance_id = info().instance_id()
     if not volume_id:
         device = device_from_mount_path(mount_path)
-        if "/nvme" in device:
-            proc = Popen(["nvme", "id-ctrl", device], stdout=PIPE, stderr=PIPE)
-            out = proc.communicate()[0]
-            for nvme_line in out.split("\n"):
-                if nvme_line.startswith("sn"):
-                    volume_id = nvme_line.split()[2]
-                    if "vol-" not in volume_id:
-                        volume_id = volume_id.replace("vol", "vol-")
-                    break
-        else:
-            volume = ec2().describe_volumes(Filters=[{"Name": "attachment.device",
-                                                      "Values": [device]},
-                                                     {"Name": "attachment.instance-id",
-                                                      "Values": [instance_id]}])
-            volume_id = volume['Volumes'][0]['VolumeId']
+        volume_id = volume_id_from_device(device)
     proc = Popen(["umount", "-f", mount_path], stdout=PIPE, stderr=PIPE)
     out = proc.communicate()[0]
     ec2().detach_volume(VolumeId=volume_id, InstanceId=instance_id)
@@ -334,6 +320,24 @@ def detach_volume(mount_path, delete_volume=False, volume_id=None):
     info().clear_cache()
 
 
+def volume_id_from_device(device):
+    if "/nvme" in device:
+        proc = Popen(["nvme", "id-ctrl", device], stdout=PIPE, stderr=PIPE)
+        out = proc.communicate()[0]
+        for nvme_line in out.split("\n"):
+            if nvme_line.startswith("sn"):
+                volume_id = nvme_line.split()[2]
+                if "vol-" not in volume_id:
+                    volume_id = volume_id.replace("vol", "vol-")
+                break
+    else:
+        instance_id = info().instance_id()
+        volume = ec2().describe_volumes(Filters=[{"Name": "attachment.device",
+                                                    "Values": [device]},
+                                                    {"Name": "attachment.instance-id",
+                                                    "Values": [instance_id]}])
+        volume_id = volume['Volumes'][0]['VolumeId']
+
 def create_snapshot(tag_key, tag_value, mount_path, wait=False, tags={},
                     copytags=[], ignore_missing_copytags=False):
     create_tags = _create_tag_array(tag_key, tag_value, tags, copytags,
